/*
** This file is part of the C Utility Toolkit (CUTK)
** Copyright (C) 2002-2005 Chris Osgood
** http://www.cutk.org
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
** Please report all bugs and problems to "bugs at cutk.org".
*/

/*
**  $Id: plugin.h 29 2005-01-12 16:31:26Z chris $
*/

#ifndef __INCLUDE_PLUGIN_H__
#define __INCLUDE_PLUGIN_H__

#include <cutk/dll.h>
#include <cutk/strhmap.h>
#include <limits.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
#define PATH_MAX MAX_PATH
#endif

/* Callback funtions should return one of these */
typedef enum PluginResult
{
   PLUGIN_NEXT,
   PLUGIN_STOP,
   PLUGIN_AGAIN
} PluginResult;

typedef PluginResult (*PluginCallback)(size_t,void*,void*);

typedef struct CallbackEntry
{
   PluginCallback callback;
   unsigned char priority;
} CallbackEntry;

struct PluginManager;

typedef struct Plugin
{
   Dll dll_;
   char path_[PATH_MAX];
   int (*initialize)(struct PluginManager*);
   int (*finalize)();
} Plugin;

typedef struct PluginManager
{
   Plugin* plugins_;             /* Array of plugins */
   StrHmap* event_map_;          /* Hash-map of indexes into callbacks_ */
   CallbackEntry** callbacks_;  /* Array of pointers to PluginEntry arrays */

   /* Plugins should use this API to access the plugin manager */
      int (*AddPluginObj)   (struct PluginManager*, Plugin*);
      int (*AddPlugin)      (struct PluginManager*, const char*);
      int (*AddDir)         (struct PluginManager*, const char* path, char recursive);
   size_t (*AddEvent)       (struct PluginManager*, const char*, PluginCallback, unsigned char);
      int (*RemoveEvent)    (struct PluginManager*, size_t, PluginCallback, unsigned char);
      int (*RemoveEventName)(struct PluginManager*, const char*, PluginCallback, unsigned char);
   size_t (*FindEvent)      (struct PluginManager*, const char*);
      int (*DoEvent)        (struct PluginManager*, size_t event, void*, void*);
      int (*DoEventName)    (struct PluginManager*, const char*, void*, void*);
} PluginManager;

/* PluginManager API */
int PluginManagerAlloc(PluginManager* mgr);
void PluginManagerFree(PluginManager* mgr);
int PluginManagerAddPluginObj(PluginManager* mgr, Plugin* plugin);
int PluginManagerAddPlugin(PluginManager* mgr, const char* path);
int PluginManagerAddDir(PluginManager* mgr, const char* path, char recursive);
size_t PluginManagerAddEvent(PluginManager* mgr, const char* event, PluginCallback cb, unsigned char pri);
int PluginManagerRemoveEvent(PluginManager* mgr, size_t event, PluginCallback cb, unsigned char pri);
int PluginManagerRemoveEventName(PluginManager* mgr, const char* event, PluginCallback cb, unsigned char pri);
size_t PluginManagerFindEvent(PluginManager* mgr, const char* event);
int PluginManagerDoEvent(PluginManager* mgr, size_t event, void* param1, void* param2);
int PluginManagerDoEventName(PluginManager* mgr, const char* event, void* param1, void* param2);

#ifdef __cplusplus
}
#endif

#endif /* __INCLUDE_PLUGIN_H__ */
