/*
** This file is part of the C Utility Toolkit (CUTK)
** Copyright (C) 2002-2005 Chris Osgood
** http://www.cutk.org
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
** Please report all bugs and problems to "bugs at cutk.org".
*/

/*
**  $Id: timer.h 7 2005-01-02 01:02:22Z chris $
*/

#ifndef __INCLUDE_TIMER_H__
#define __INCLUDE_TIMER_H__

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#else
#include <sys/time.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Timer mode
 */
typedef enum TimerMode
{
   TIMER_TICKS,
   TIMER_SECS,
   TIMER_FPS
} TimerMode;

/**
 * Timer object
 */
typedef struct Timer
{
#ifdef _WIN32
   LARGE_INTEGER tm;
   LARGE_INTEGER freq;
   LARGE_INTEGER elapsed;
#else
   struct timeval tm;
   unsigned long freq;
   unsigned long elapsed;
#endif
   char stopped;
} Timer;

/* Function prototypes */

void TimerStart(Timer* t);
double TimerElapsed(Timer* t, TimerMode mode);
double TimerEnd(Timer* t, TimerMode mode);

#ifdef __cplusplus
}
#endif

#endif /* __INCLUDE_TIMER_H__ */

