/*
** This file is part of the C Utility Toolkit (CUTK)
** Copyright (C) 2002-2005 Chris Osgood
** http://www.cutk.org
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
** Please report all bugs and problems to "bugs at cutk.org".
*/

/*
** $Id: util.h 7 2005-01-02 01:02:22Z chris $
*/

#ifndef _UTIL_H
#define _UTIL_H

#include <cutk/cutk.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

void Byte2Binary(char* out, unsigned char c);
unsigned char Binary2Byte(const char* in);
void Byte2Hex(char* out, unsigned char c);
unsigned char Hex2Byte(const char* in);
char* Bin2Hex(const unsigned char* in, size_t len);
unsigned char* Hex2Bin(const char* in, size_t width);

/* Inline functions */

/* Fast string equality test.  In most cases this is faster than strcmp */
INLINE char StrEq(const char* s1, const char* s2)
{
   if (s1 == s2) return 1;
   if (!s1 || !s2) return 0;
   while (*s1 != '\0' && *s1 == *s2)
      s1++, s2++;
   return *s1 == *s2;
}

/* Fast string equality test with limited length */
INLINE char StrEqN(const char* s1, const char* s2, size_t len)
{
   size_t i = 0;
   if (s1 == s2) return 1;
   if (!s1 || !s2) return 0;
   while (*s1 != '\0' && *s1 == *s2 && i < len)
      s1++, s2++;
   return *s1 == *s2;
}

#ifdef __cplusplus
}
#endif

#endif /* _UTIL_H */
