/*
** This file is part of the C Utility Toolkit (CUTK)
** Copyright (C) 2002-2005 Chris Osgood
** http://www.cutk.org
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
** Please report all bugs and problems to "bugs at cutk.org".
*/

/*
** See memory.h for more information
**
** $Id: memory.c 31 2005-01-12 17:31:03Z chris $
*/

#include <cutk/memory.h>

/**
 * Proxy for standard calloc() library function
 * DO NOT MAKE INLINE
 * @param nmemb number of members
 * @param size of each item
 */
void* Calloc(size_t nmemb, size_t size)
{
   return calloc(nmemb, size);
}

/**
 * Proxy for standard malloc() library function
 * DO NOT MAKE INLINE
 * @param size of memory block
 */
void* Malloc(size_t size)
{
   return malloc(size);
}

/**
 * Proxy for standard realloc() library function
 * DO NOT MAKE INLINE
 * @param ptr pointer to memory block
 * @param size new size of memory block
 */
void* Realloc(void* ptr, size_t size)
{
   return realloc(ptr, size);
}

/**
 * Proxy for standard free() library function
 * DO NOT MAKE INLINE
 * @param ptr pointer to memory block
 */
void Free(void* ptr)
{
   free(ptr);
}

/**
 * Returns the amount of memory being used by the current process.
 * The psapi library (psapi.lib) must be linked in on Windows.
 */
#ifdef _WIN32
#include <windows.h>
#include <psapi.h>
size_t MemUsage(void)
{
   PROCESS_MEMORY_COUNTERS mem;
   HANDLE hProc = OpenProcess(PROCESS_ALL_ACCESS, FALSE, GetCurrentProcessId());
   if (!hProc)
      return 0;

   GetProcessMemoryInfo(hProc, &mem, sizeof(mem));
   CloseHandle(hProc);

   return mem.WorkingSetSize;
}
#else /* Linux */
#include <stdio.h>
#include <string.h>
size_t MemUsage(void)
{
   FILE* fp;
   char buf[256];
   char* c;
   int i;

   fp = fopen("/proc/self/stat", "rb");
   if (!fp)
      return 0;

   fgets(buf, sizeof(buf), fp);
   fclose(fp);

   c = strrchr(buf, ')');
   for (i = 0; i < 21 && c != 0; i++)
      c = strchr(++c, 0x20);

   if (c != 0)
      return atoi(c);
   else
      return 0;
}
#endif /* MemUsage */

